/*
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ,
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
*/

package at.asitplus.eidas.specific.connector.builder;

import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.w3c.dom.DOMException;

import at.asitplus.eidas.specific.connector.MsEidasNodeConstants;
import at.gv.egiz.eaaf.core.api.IRequest;
import at.gv.egiz.eaaf.core.api.idp.IAuthData;
import at.gv.egiz.eaaf.core.api.idp.ISpConfiguration;
import at.gv.egiz.eaaf.core.api.idp.auth.data.IAuthProcessDataContainer;
import at.gv.egiz.eaaf.core.exceptions.EaafAuthenticationException;
import at.gv.egiz.eaaf.core.exceptions.EaafBuilderException;
import at.gv.egiz.eaaf.core.exceptions.EaafConfigurationException;
import at.gv.egiz.eaaf.core.exceptions.EaafException;
import at.gv.egiz.eaaf.core.exceptions.EaafParserException;
import at.gv.egiz.eaaf.core.exceptions.XPathException;
import at.gv.egiz.eaaf.core.impl.data.Pair;
import at.gv.egiz.eaaf.core.impl.idp.AuthenticationData;
import at.gv.egiz.eaaf.core.impl.idp.auth.builder.AbstractAuthenticationDataBuilder;
import at.gv.egiz.eaaf.core.impl.idp.auth.data.AuthProcessDataWrapper;

@Service("AuthenticationDataBuilder")
public class AuthenticationDataBuilder extends AbstractAuthenticationDataBuilder {
  private static final Logger log = LoggerFactory.getLogger(AuthenticationDataBuilder.class);

  @Override
  public IAuthData buildAuthenticationData(IRequest pendingReq) throws EaafAuthenticationException {

    final IAuthProcessDataContainer authProcessData = pendingReq.getSessionData(AuthProcessDataWrapper.class);
    final AuthenticationData authData = new AuthenticationData();

    try {
      generateDeprecatedBasicAuthData(authData, pendingReq, authProcessData);

      // set specific informations
      authData.setSsoSessionValidTo(new Date(new Date().getTime()
          + MsEidasNodeConstants.DEFAULT_PVP_ASSERTION_VALIDITY * 60 * 1000));

    } catch (EaafBuilderException | EaafParserException | EaafConfigurationException
        | XPathException | DOMException e) {
      log.warn("Can not build authentication data from auth. process information");
      throw new EaafAuthenticationException("builder.11", new Object[] { e.getMessage() }, e);

    }

    return authData;

  }

  @Override
  protected IAuthData buildDeprecatedAuthData(IRequest arg0) throws EaafException {
    return new AuthenticationData();
    
  }

  @Override
  protected void buildServiceSpecificAuthenticationData(IAuthData arg0, IRequest arg1) throws EaafException {
    // TODO Auto-generated method stub

  }

  @Override
  protected IAuthData getAuthDataInstance(IRequest arg0) throws EaafException {
    return new AuthenticationData();

  }

  @Override
  protected Pair<String, String> buildOAspecificbPK(IRequest pendingReq, AuthenticationData authData)
      throws EaafBuilderException {
    // TODO: check if bPK already exists

    return super.buildOAspecificbPK(pendingReq, authData);

  }

  @Override
  protected Pair<String, String> getEncryptedBpkFromPvpAttribute(IAuthProcessDataContainer arg0,
      AuthenticationData arg1, ISpConfiguration arg2) throws EaafBuilderException {
    return null;

  }

  @Override
  protected Pair<String, String> getbaseIdFromSzr(AuthenticationData arg0, String arg1, String arg2) {
    return null;

  }

}
