/*
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ,
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
*/

package at.asitplus.eidas.specific.connector.logger;

import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import at.asitplus.eidas.specific.connector.MsEidasNodeConstants;
import at.gv.egiz.components.eventlog.api.Event;
import at.gv.egiz.components.eventlog.api.EventConstants;
import at.gv.egiz.components.eventlog.api.EventLogFactory;
import at.gv.egiz.components.eventlog.api.EventLoggingException;
import at.gv.egiz.eaaf.core.api.IRequest;
import at.gv.egiz.eaaf.core.api.idp.IConfiguration;
import at.gv.egiz.eaaf.core.api.idp.ISpConfiguration;
import at.gv.egiz.eaaf.core.api.logging.IRevisionLogger;

public class RevisionLogger extends EventLogFactory implements IRevisionLogger {
  private static final Logger log = LoggerFactory.getLogger(RevisionLogger.class);

  @Autowired
  private IConfiguration basicConfig;

  @Override
  public void logEvent(ISpConfiguration oaConfig, int eventCode, String message) {
    logEvent(createNewEvent(new Date().getTime(), eventCode, message));

  }

  @Override
  public void logEvent(IRequest pendingRequest, int eventCode) {
    logEvent(createNewEvent(new Date().getTime(), eventCode,
        pendingRequest.getUniqueSessionIdentifier(), pendingRequest.getUniqueTransactionIdentifier()));

  }

  @Override
  public void logEvent(IRequest pendingRequest, int eventCode, String message) {
    logEvent(createNewEvent(new Date().getTime(), eventCode, message,
        pendingRequest.getUniqueSessionIdentifier(), pendingRequest.getUniqueTransactionIdentifier()));

  }

  @Override
  public void logEvent(int eventCode, String message) {
    logEvent(createNewEvent(new Date().getTime(), eventCode, message));

  }

  @Override
  public void logEvent(String sessionID, String transactionID, int eventCode, String message) {
    logEvent(createNewEvent(new Date().getTime(), eventCode, message, sessionID, transactionID));

  }

  @Override
  public void logEvent(String sessionID, String transactionID, int eventCode) {
    logEvent(createNewEvent(new Date().getTime(), eventCode, sessionID, transactionID));

  }

  private void logEvent(Event event) {
    try {
      if (event.getEventCode() >= 1100) {
        if (event.getEventCode() == EventConstants.TRANSACTION_IP
            && !basicConfig.getBasicConfigurationBoolean(
                MsEidasNodeConstants.PROP_CONFIG_REVISIONLOG_LOG_IP_ADDRESS_OF_USER, true)) {
          log.trace("Ignore Event: " + event.getEventCode() + " because IP adresse logging prohibited");
          return;

        }

        getEventLog().logEvent(event);

      } else {
        log.trace("Ignore Event: " + event.getEventCode()
            + " because session functionallity is not implemented");
      }

    } catch (final EventLoggingException e) {
      log.warn("Event logging FAILED! Reason: " + e.getMessage());

    }

  }
}
