/*
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ,
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
*/

package at.asitplus.eidas.specific.connector.processes.tasks;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import at.asitplus.eidas.specific.connector.MsConnectorEventCodes;
import at.asitplus.eidas.specific.connector.MsEidasNodeConstants;
import at.asitplus.eidas.specific.connector.gui.StaticGuiBuilderConfiguration;
import at.gv.egiz.eaaf.core.api.gui.IGuiBuilderConfiguration;
import at.gv.egiz.eaaf.core.api.gui.ISpringMvcGuiFormBuilder;
import at.gv.egiz.eaaf.core.api.idp.IConfiguration;
import at.gv.egiz.eaaf.core.api.idp.process.ExecutionContext;
import at.gv.egiz.eaaf.core.exceptions.EaafException;
import at.gv.egiz.eaaf.core.exceptions.GuiBuildException;
import at.gv.egiz.eaaf.core.exceptions.TaskExecutionException;
import at.gv.egiz.eaaf.core.impl.idp.auth.modules.AbstractAuthServletTask;

/**
 * Create country-selection page.
 * 
 * @author tlenz
 *
 */
@Component("GenerateCountrySelectionFrameTask")
public class GenerateCountrySelectionFrameTask extends AbstractAuthServletTask {
  private static final Logger log = LoggerFactory.getLogger(GenerateCountrySelectionFrameTask.class);

  @Autowired
  ISpringMvcGuiFormBuilder guiBuilder;
  @Autowired
  IConfiguration basicConfig;


  @Override
  public void execute(ExecutionContext executionContext, HttpServletRequest request,
      HttpServletResponse response)
      throws TaskExecutionException {
    try {
      revisionsLogger.logEvent(pendingReq, MsConnectorEventCodes.STARTING_COUNTRY_SELECTION);

      final IGuiBuilderConfiguration config = new StaticGuiBuilderConfiguration(
          basicConfig,
          pendingReq,
          basicConfig.getBasicConfiguration(
              MsEidasNodeConstants.PROP_CONFIG_WEBCONTENT_TEMPLATES_CCSELECTION, 
              MsEidasNodeConstants.TEMPLATE_HTML_COUNTRYSELECTION),
          MsEidasNodeConstants.ENDPOINT_COUNTRYSELECTION,
          resourceLoader);

      guiBuilder.build(request, response, config, "BKU-Selection form");

    } catch (final GuiBuildException e) {
      log.warn("Can not build GUI:'BKU-Selection'. Msg:" + e.getMessage());
      throw new TaskExecutionException(pendingReq,
          "Can not build GUI. Msg:" + e.getMessage(),
          new EaafException("gui.00", new Object[] { e.getMessage() }, e));

    } catch (final Exception e) {
      log.warn("FinalizeAuthenticationTask has an internal error", e);
      throw new TaskExecutionException(pendingReq, e.getMessage(), e);

    }

  }

}
