/*
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ,
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
*/

package at.asitplus.eidas.specific.connector.gui;

import java.io.InputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import at.gv.egiz.eaaf.core.api.gui.IVelocityGuiBuilderConfiguration;
import at.gv.egiz.eaaf.core.exceptions.GuiBuildException;
import at.gv.egiz.eaaf.core.impl.gui.AbstractVelocityGuiFormBuilderImpl;

@Service("DefaultVelocityGUIBuilderImpl")
public class DefaultVelocityGuiBuilderImpl extends AbstractVelocityGuiFormBuilderImpl {
  private static final Logger log = LoggerFactory.getLogger(DefaultVelocityGuiBuilderImpl.class);

  private static final String CLASSPATH_HTMLTEMPLATES_DIR = "templates/";

  public DefaultVelocityGuiBuilderImpl() throws GuiBuildException {
    super();

  }

  @Override
  protected InputStream getInternalTemplate(IVelocityGuiBuilderConfiguration config)
      throws GuiBuildException {
    final String viewName = config.getViewName();
    log.debug("GUI template:" + viewName + " is not found in configuration directory. "
        + " Load template from project library ... ");
    final String pathLocation = getInternalClasspathTemplateDir(config, CLASSPATH_HTMLTEMPLATES_DIR)
        + viewName;
    try {
      final InputStream is = Thread.currentThread().getContextClassLoader().getResourceAsStream(pathLocation);
      return is;

    } catch (final Exception e1) {
      log.error("GUI template:" + pathLocation + " is NOT loadable  from classpath!", e1);
      throw new GuiBuildException("GUI template:" + pathLocation + " is NOT loadable from classpath!", e1);

    }
  }

}
