/*
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ,
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
 */

package at.asitplus.eidas.specific.modules.auth.eidas.v2.service;

import java.util.Map;

import at.asitplus.eidas.specific.modules.auth.eidas.v2.dao.ErnbEidData;
import at.asitplus.eidas.specific.modules.auth.eidas.v2.exception.EidasAttributeException;
import at.asitplus.eidas.specific.modules.auth.eidas.v2.exception.EidPostProcessingException;
import at.gv.egiz.eaaf.core.api.IRequest;
import eu.eidas.auth.commons.light.ILightRequest;
import eu.eidas.auth.commons.light.impl.LightRequest.Builder;

public interface ICcSpecificEidProcessingService {

  /**
   * Post-process eIDAS eID attributes into national format.
   * 
   * @param eidasAttrMap Map of eIDAS attributes in format friendlyName and
   *                     attribute
   * 
   * @return eID attributes for SZR request
   * @throws EidPostProcessingException In case of a post-processing error
   * @throws EidasAttributeException In case of an invalid eIDAS attribute value
   */
  ErnbEidData postProcess(Map<String, Object> eidasAttrMap) throws EidPostProcessingException,
      EidasAttributeException;

  /**
   * Pre Process eIDAS request into national requirements.
   * 
   * @param selectedCC          Citizen Country from selection
   * @param pendingReq          current pending request
   * @param authnRequestBuilder eIDAS {@link ILightRequest} builder
   * @throws EidPostProcessingException In case of a pre-processing error
   */
  void preProcess(String selectedCC, IRequest pendingReq, Builder authnRequestBuilder)
      throws EidPostProcessingException;

}
