/*
 * Copyright 2018 A-SIT Plus GmbH
 * AT-specific eIDAS Connector has been developed in a cooperation between EGIZ,
 * A-SIT Plus GmbH, A-SIT, and Graz University of Technology.
 *
 * Licensed under the EUPL, Version 1.2 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "License");
 * You may not use this work except in compliance with the License.
 * You may obtain a copy of the License at:
 * https://joinup.ec.europa.eu/news/understanding-eupl-v12
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * This product combines work with different licenses. See the "NOTICE" text
 * file for details on the various modules and licenses.
 * The "NOTICE" text file is part of the distribution. Any derivative works
 * that you distribute must include a readable copy of the "NOTICE" text file.
*/

package at.asitplus.eidas.specific.modules.auth.eidas.v2.test;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.annotation.DirtiesContext.ClassMode;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import at.asitplus.eidas.specific.modules.auth.eidas.v2.exception.EidPostProcessingException;
import at.asitplus.eidas.specific.modules.auth.eidas.v2.service.CcSpecificEidProcessingService;
import at.gv.egiz.eaaf.core.api.data.EaafConfigConstants;
import at.gv.egiz.eaaf.core.api.idp.IConfigurationWithSP;
import at.gv.egiz.eaaf.core.impl.idp.module.test.DummySpConfiguration;
import at.gv.egiz.eaaf.core.impl.idp.module.test.TestRequestImpl;
import eu.eidas.auth.commons.light.impl.LightRequest;
import eu.eidas.auth.commons.light.impl.LightRequest.Builder;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration("/SpringTest-context_tasks_test.xml")
@DirtiesContext(classMode = ClassMode.AFTER_CLASS)
public class EidasRequestPreProcessingSecondTest {

  @Autowired
  private IConfigurationWithSP basicConfig;
  @Autowired
  private CcSpecificEidProcessingService preProcessor;

  private TestRequestImpl pendingReq;
  private DummySpConfiguration oaParam;
  private Builder authnRequestBuilder;

  /**
   * jUnit class initializer.
   * 
   * @throws IOException In case of an error
   */
  @BeforeClass
  public static void classInitializer() throws IOException {
    final String current = new java.io.File(".").toURI().toString();
    System.setProperty("eidas.ms.configuration", current
        + "src/test/resources/config/junit_config_1.properties");

  }

  /**
   * jUnit test set-up.
   * 
   */
  @Before
  public void setUp() {

    final Map<String, String> spConfig = new HashMap<>();
    spConfig.put(EaafConfigConstants.SERVICE_UNIQUEIDENTIFIER, "testSp");
    spConfig.put("target", "urn:publicid:gv.at:cdid+XX");
    oaParam = new DummySpConfiguration(spConfig, basicConfig);

    pendingReq = new TestRequestImpl();
    pendingReq.setSpConfig(oaParam);
    pendingReq.setPendingReqId(at.gv.egiz.eaaf.core.impl.utils.Random.nextProcessReferenceValue());
    pendingReq.setAuthUrl("http://test.com/");

    authnRequestBuilder = LightRequest.builder();
    authnRequestBuilder.id(UUID.randomUUID().toString());
    authnRequestBuilder.issuer("Test");

  }

  @Test
  public void prePreProcessDeUnknownAttribute() throws EidPostProcessingException {

    final String testCountry = "DE";
    authnRequestBuilder.citizenCountryCode(testCountry);
    preProcessor.preProcess(testCountry, pendingReq, authnRequestBuilder);

    final LightRequest lightReq = authnRequestBuilder.build();

    Assert.assertEquals("ProviderName is not Static", "myNode", lightReq.getProviderName());
    Assert.assertEquals("no PublicSP", "public", lightReq.getSpType());
    Assert.assertEquals("Requested attribute size not match", 8, lightReq.getRequestedAttributes().size());

  }

}
